﻿/****************************************************************************

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

-- Copyright 2009 Terence Tsang
-- admin@shinedraw.com
-- http://www.shinedraw.com
-- Your Flash vs Silverlight Repositry

****************************************************************************/


using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Windows.Media.Imaging;

/*
*	A Water Effect Demonstratoin in C#
*   from shinedraw.com
*/

namespace WaterEffect
{
    public partial class WaterEffect : UserControl
    {
		private const string ImagePath = "images/image1.jpg";		// Image source
        private const double ImageWidth = 550;    // Image Width
        private const double ImageHeight = 400;   // Image Height
        private const double InitialRadius = 5;	// Initial Water Radius
        private const double RadiusIncrement = 4;		// Radius Increment
        private const int LayerCount = 15;			// Number of Layer to simulate the drop
        private const double LayerScale = 0.05;		// Image Scale Up
        private const double Spring = 1;		// Control the Water Movement Event
        private const double SpringDrag = 0.85;// Springness Decrement
        private const int FPS = 24;                // fps of the on enter frame event

        private Image[] images = new Image[LayerCount];		// Stored the added images
        private Point targetPoint = new Point(ImageWidth / 2, ImageHeight / 2);	// Store the Mouse Position


        public WaterEffect()
        {
            InitializeComponent();

            addImage();

            this.MouseMove += new MouseEventHandler(WaterEffect_MouseMove);

            App.Current.Host.Settings.MaxFrameRate = FPS;
            CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
        }

        /////////////////////////////////////////////////////        
        // Handlers
        /////////////////////////////////////////////////////	

        void WaterEffect_MouseMove(object sender, MouseEventArgs e)
        {
            targetPoint.X = e.GetPosition(this).X;
            targetPoint.Y = e.GetPosition(this).Y;
        }

        void CompositionTarget_Rendering(object sender, EventArgs e)
        {
			double springness = Spring;
            double scale = 1;
			
			for(int i = 0; i < images.Length; i++){
				// update the mask position
				Image image  = images[i];

                // reposition the image according to the target point
                double offsetX = -(image.Width - ImageWidth) / ImageWidth * targetPoint.X;
                double offsetY = -(image.Height - ImageHeight) / ImageHeight * targetPoint.Y;
                image.SetValue(Canvas.LeftProperty, offsetX);
                image.SetValue(Canvas.TopProperty, offsetY);

                // update the mask
                EllipseGeometry ellipseGeometry = (EllipseGeometry)image.Clip;
                Point center = ellipseGeometry.Center;
                center.X = center.X + (targetPoint.X - offsetX  - center.X) * springness;
                center.Y = center.Y + (targetPoint.Y - offsetY  - center.Y) * springness;
                ellipseGeometry.Center = center;
                image.Clip = ellipseGeometry;
				
				springness *= SpringDrag;
                scale += LayerScale;
			}
        }

        /////////////////////////////////////////////////////        
        // Private Methods
        /////////////////////////////////////////////////////	


        private void addImage()
        {
			// get the image resources
			
			// init variables
			double radius = InitialRadius;
			double scale = 1;

            ImageSource imageSource = new BitmapImage(new Uri(ImagePath, UriKind.Relative));

			for(int i = 0; i < LayerCount; i++){
				// create the mask
                // clip the image
                EllipseGeometry ellipseGeometry = new EllipseGeometry();
                ellipseGeometry.Center = new Point(0 , 0);
                ellipseGeometry.RadiusX = radius;
                ellipseGeometry.RadiusY = radius;

                double offsetX = ImageWidth * (1 -  scale) / 2;
                double offestY = ImageHeight * (1 -  scale) / 2;

				// creat the image
	            Image image = new Image();
                image.Source = imageSource;
                image.Width = ImageWidth * scale;
                image.Height = ImageHeight * scale;
				image.SetValue(Canvas.LeftProperty, offsetX);
				image.SetValue(Canvas.TopProperty, offestY);
				
				// mask the image
                image.Clip = ellipseGeometry;
				
				// add the mask and images to the stage
                images[i] = image;
                LayoutRoot.Children.Insert(0, image);
				
				scale += LayerScale;
				radius += RadiusIncrement;
			}
			
			// add the image background
            Image background = new Image();
            background.Source = imageSource;
            LayoutRoot.Children.Insert(0, background);
        }
    }
}
